@@ -9,7 +9,6 @@ from abc import ABCMeta, abstractmethod, abstractproperty
 
 from builtins import object
 from datetime import date, datetime
-
 from future.moves import builtins
 from future.types import newint, newlist, newstr
 from future.utils import with_metaclass
@@ -71,6 +70,15 @@ class AbstractRestriction(tuple):
     def default(self):
         return self._default
 
+    @property
+    def schema_value(self):
+        """
+        Provide a string representation of valid values.
+        Useful for generating API-like documentation
+        :rtype: str or dict or list
+        """
+        raise NotImplementedError('schema value is not defined.')
+
     # NOTE:
     # Opportunity 1: ES restriction building capability can be triggered from es_ables.
     # Opportunity 2: The encoder system can be abstracted to support multiple encoding needs (including custom).
@@ -105,9 +113,13 @@ class AbstractRestriction(tuple):
         return tuple(self)
 
     def __deepcopy__(self, memodict={}):
-        memodict[id(self)] = self.__copy__()
+        """
+        NOTE: `deepcopy` uses memoization to store a previously copied instance.
+        """
         _r = copy.deepcopy(self.__copy__())
-        return self.__class__(_r[0], default=_r[1])
+        instance = self.__class__(_r[0], default=_r[1])
+        memodict[id(self)] = instance
+        return instance
 
 
 def is_immutable(obj):
@@ -153,6 +165,15 @@ class SingletonRestriction(AbstractRestriction):
             cls._cache[hashable] = super(SingletonRestriction, cls).__new__(cls, restriction_tuple)
             return cls._cache[hashable]
 
+    @property
+    def schema_value(self):
+        """
+        Provide a string representation of valid values.
+        Useful for generating API-like documentation
+        :rtype: str or dict or list
+        """
+        raise NotImplementedError('schema value is not defined.')
+
     @property
     def es_restrictions(self):
         raise NotImplementedError('ES restrictions not defined.')
@@ -201,6 +222,16 @@ class _ListTypeRestriction(SingletonRestriction):
             raise RestrictionError.bad_data(type(data), self._allowed)
         return data
 
+    @property
+    def schema_value(self):
+        """
+        :rtype: str
+        """
+        return ' or '.join({
+            x.__name__ for x in self.allowed
+            if x.__name__ not in ['newstr', 'newint', 'long']  # Exclude Py2/3 compatibility
+        })
+
     @property
     def es_restrictions(self):
         _x = set()
@@ -250,6 +281,13 @@ class _ListValueRestriction(SingletonRestriction):
             raise RestrictionError.bad_data(data, self._allowed)
         return data
 
+    @property
+    def schema_value(self):
+        """
+        :rtype: str
+        """
+        return ' or '.join({'"%s"' % x for x in self.allowed})
+
     @property
     def es_restrictions(self):
         return ESEncoder.default('keyword')
@@ -294,6 +332,13 @@ class _ListNoRestriction(SingletonRestriction):
     def __call__(self, data, **kwargs):
         return data
 
+    @property
+    def schema_value(self):
+        """
+        :rtype: str
+        """
+        return 'any'
+
     @property
     def es_restrictions(self):
         return None
@@ -340,6 +385,14 @@ class _MgdRestRestriction(AbstractRestriction):
     def __call__(self, data, **kwargs):
         return self._allowed(data)
 
+    # TODO: This needs more finesse - support ManagedList
+    @property
+    def schema_value(self):
+        """
+        :rtype: str
+        """
+        return self.allowed.schema_value
+
     @property
     def es_restrictions(self):
         # NOTE: _allowed is of ManagedRestrictions type. _restriction is of AbstractRestriction type
@@ -392,6 +445,13 @@ class _NullableDataObjectRestriction(SingletonRestriction):
     def dataobj(self):
         return self.allowed
 
+    @property
+    def schema_value(self):
+        """
+        :rtype: dict
+        """
+        return self.dataobj.schema
+
     @property
     def es_restrictions(self):
         if hasattr(self.dataobj, 'es_restrictions'):
@@ -591,6 +651,13 @@ class ManagedRestrictions(object, with_metaclass(ABCMeta)):
         """
         return self._restriction.default
 
+    @property
+    def schema_value(self):
+        """
+        :rtype: str or dict or list
+        """
+        return self._restriction.schema_value
+
     def __call__(self, value, strict=True):
         """
         Entry point for data management. DataObject will trigger a call to __call__ when it encounters a managed
